@@ -27,10 +27,13 @@ import org.apache.ratis.util.StringUtils;
 import org.apache.ratis.util.TimeDuration;
 import org.apache.ratis.util.function.CheckedRunnable;
 import org.junit.After;
-import org.junit.Assert;
 import org.junit.Rule;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.TestInfo;
+import org.junit.jupiter.api.Timeout;
 import org.junit.rules.TestName;
-import org.junit.rules.Timeout;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.slf4j.event.Level;
@@ -47,6 +50,7 @@ import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.Supplier;
 
+@Timeout(value = 100)
 public abstract class BaseTest {
   public final Logger LOG = LoggerFactory.getLogger(getClass());
 
@@ -80,7 +84,24 @@ public abstract class BaseTest {
     return peersWithPriority;
   }
 
+
+  /*
+   * Junit 4 reference will be removed and the code will be refactored once
+   * all the unit tests are migrated to Junit 5.
+   */
+
+  private String testCaseName;
+
+  @BeforeEach
+  public void setup(TestInfo testInfo) {
+    testCaseName = testInfo.getTestMethod()
+        .orElseThrow(() -> new RuntimeException("Exception while getting test name."))
+        .getName();
+  }
+
+  // @After annotation is retained to support junit 4 tests.
   @After
+  @AfterEach
   public void assertNoFailures() {
     final Throwable e = firstException.get();
     if (e != null) {
@@ -90,9 +111,12 @@ public abstract class BaseTest {
     ExitUtils.assertNotTerminated();
   }
 
+  // Retained to support junit 4 tests.
   @Rule
-  public final Timeout globalTimeout = new Timeout(getGlobalTimeoutSeconds(), TimeUnit.SECONDS );
+  public final org.junit.rules.Timeout globalTimeout = new org.junit.rules.Timeout(
+      getGlobalTimeoutSeconds(), TimeUnit.SECONDS );
 
+  // Retained to support junit 4 tests.
   @Rule
   public final TestName testName = new TestName();
 
@@ -100,7 +124,7 @@ public abstract class BaseTest {
     return 100;
   }
 
-  private static final Supplier<File> rootTestDir = JavaUtils.memoize(
+  private static final Supplier<File> ROOT_TEST_DIR = JavaUtils.memoize(
       () -> JavaUtils.callAsUnchecked(() -> {
         final File dir = new File(System.getProperty("test.build.data", "target/test/data"),
             Integer.toHexString(ThreadLocalRandom.current().nextInt()));
@@ -114,7 +138,7 @@ public abstract class BaseTest {
 
 
   public static File getRootTestDir() {
-    return rootTestDir.get();
+    return ROOT_TEST_DIR.get();
   }
 
   public File getClassTestDir() {
@@ -122,7 +146,9 @@ public abstract class BaseTest {
   }
 
   public File getTestDir() {
-    return new File(getClassTestDir(), testName.getMethodName());
+    // This will work for both junit 4 and 5.
+    final String name = testCaseName != null ? testCaseName : testName.getMethodName();
+    return new File(getClassTestDir(), name);
   }
 
   @SafeVarargs
@@ -135,13 +161,13 @@ public abstract class BaseTest {
           description, expectedThrowableClass.getSimpleName(),
           StringUtils.array2String(expectedCauseClasses, Class::getSimpleName));
     }
-    Assert.assertEquals(expectedThrowableClass, t.getClass());
+    Assertions.assertEquals(expectedThrowableClass, t.getClass());
 
     for (Class<? extends Throwable> expectedCause : expectedCauseClasses) {
       final Throwable previous = t;
       t = Objects.requireNonNull(previous.getCause(),
           () -> "previous.getCause() == null for previous=" + previous);
-      Assert.assertEquals(expectedCause, t.getClass());
+      Assertions.assertEquals(expectedCause, t.getClass());
     }
   }
 
