@@ -46,6 +46,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Optional;
 import java.util.concurrent.Callable;
+import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorCompletionService;
 import java.util.concurrent.ExecutorService;
@@ -143,7 +144,7 @@ class LeaderElection implements Runnable {
     }
 
     void shutdown() {
-      executor.shutdown();
+      executor.shutdownNow();
     }
 
     void submit(Callable<RequestVoteReplyProto> task) {
@@ -183,6 +184,7 @@ class LeaderElection implements Runnable {
   private final String name;
   private final LifeCycle lifeCycle;
   private final Daemon daemon;
+  private final CompletableFuture<Void> stopped = new CompletableFuture<>();
 
   private final RaftServerImpl server;
   private final boolean skipPreVote;
@@ -223,8 +225,9 @@ class LeaderElection implements Runnable {
     }
   }
 
-  void shutdown() {
+  CompletableFuture<Void> shutdown() {
     lifeCycle.checkStateAndClose();
+    return stopped;
   }
 
   @VisibleForTesting
@@ -234,6 +237,14 @@ class LeaderElection implements Runnable {
 
   @Override
   public void run() {
+    try {
+      runImpl();
+    } finally {
+      stopped.complete(null);
+    }
+  }
+
+  private void runImpl() {
     if (!lifeCycle.compareAndTransition(STARTING, RUNNING)) {
       final LifeCycle.State state = lifeCycle.getCurrentState();
       LOG.info("{}: skip running since this is already {}", this, state);
@@ -333,8 +344,8 @@ class LeaderElection implements Runnable {
           return true;
         case NOT_IN_CONF:
         case SHUTDOWN:
-          server.getRaftServer().close();
-          server.getStateMachine().event().notifyServerShutdown(server.getRoleInfoProto());
+          server.close();
+          server.getStateMachine().event().notifyServerShutdown(server.getRoleInfoProto(), false);
           return false;
         case TIMEOUT:
           return false; // should retry
