@@ -87,6 +87,7 @@ class ServerState {
   /**
    * Candidate that this peer granted vote for in current term (or null if none)
    */
+  @SuppressWarnings({"squid:S3077"}) // Suppress volatile for generic type
   private volatile RaftPeerId votedFor;
 
   /**
@@ -158,6 +159,7 @@ class ServerState {
   }
 
   void start() {
+    // initialize stateMachineUpdater
     stateMachineUpdater.get().start();
   }
 
@@ -170,6 +172,7 @@ class ServerState {
     }
   }
 
+  @SuppressWarnings({"squid:S2095"}) // Suppress closeable  warning
   private static RaftLog initRaftLog(RaftGroupMemberId memberId, RaftServerImpl server, RaftStorage storage,
       Consumer<LogEntryProto> logConsumer, LongSupplier getSnapshotIndexFromStateMachine,
       RaftProperties prop) throws IOException {
@@ -321,24 +324,21 @@ class ServerState {
     Objects.requireNonNull(operation.getLogEntry());
   }
 
-  /**
-   * Check if accept the leader selfId and term from the incoming AppendEntries rpc.
-   * If accept, update the current state.
-   * @return true if the check passes
-   */
-  boolean recognizeLeader(RaftPeerId peerLeaderId, long leaderTerm) {
+  /** @return true iff the given peer id is recognized as the leader. */
+  boolean recognizeLeader(Object op, RaftPeerId peerId, long peerTerm) {
     final long current = currentTerm.get();
-    if (leaderTerm < current) {
+    if (peerTerm < current) {
+      LOG.warn("{}: Failed to recognize {} as leader for {} since peerTerm = {} < currentTerm = {}",
+          getMemberId(), peerId, op, peerTerm, current);
       return false;
     }
     final RaftPeerId curLeaderId = getLeaderId();
-    if (leaderTerm > current || curLeaderId == null) {
-      // If the request indicates a term that is greater than the current term
-      // or no leader has been set for the current term, make sure to update
-      // leader and term later
-      return true;
+    if (peerTerm == current && curLeaderId != null && !curLeaderId.equals(peerId)) {
+      LOG.warn("{}: Failed to recognize {} as leader for {} since current leader is {} (peerTerm = currentTerm = {})",
+          getMemberId(), peerId, op, curLeaderId, current);
+      return false;
     }
-    return curLeaderId.equals(peerLeaderId);
+    return true;
   }
 
   static int compareLog(TermIndex lastEntry, TermIndex candidateLastEntry) {
@@ -387,6 +387,10 @@ class ServerState {
     LOG.trace("{}: {}", getMemberId(), configurationManager);
   }
 
+  void truncate(long logIndex) {
+    configurationManager.removeConfigurations(logIndex);
+  }
+
   void updateConfiguration(List<LogEntryProto> entries) {
     if (entries != null && !entries.isEmpty()) {
       configurationManager.removeConfigurations(entries.get(0).getIndex());
@@ -424,7 +428,6 @@ class ServerState {
       }
       LOG.warn(getMemberId() + ": Failed to join " + getStateMachineUpdater(), e);
     }
-    LOG.info("{}: applyIndex: {}", getMemberId(), getLastAppliedIndex());
 
     try {
       if (log.isInitialized()) {
