@@ -216,6 +216,9 @@ class StateMachineUpdater implements Runnable {
     // Thus it is possible to have applied > committed initially.
     final long applied = getLastAppliedIndex();
     for(; applied >= raftLog.getLastCommittedIndex() && state == State.RUNNING && !shouldStop(); ) {
+      if (server.getSnapshotRequestHandler().shouldTriggerTakingSnapshot()) {
+        takeSnapshot();
+      }
       if (awaitForSignal.await(100, TimeUnit.MILLISECONDS)) {
         return;
       }
