@@ -42,8 +42,10 @@ import org.apache.ratis.util.JavaUtils;
 import org.apache.ratis.util.Slf4jUtils;
 import org.apache.ratis.util.TimeDuration;
 import org.apache.ratis.util.Timestamp;
-import org.junit.Assert;
-import org.junit.Test;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.Assumptions;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.api.Timeout;
 import org.slf4j.Logger;
 import org.slf4j.event.Level;
 
@@ -94,27 +96,27 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
   }
 
   static CompletableFuture<Void> killAndRestartServer(
-      RaftPeerId id, long killSleepMs, long restartSleepMs, MiniRaftCluster cluster, Logger LOG) {
+      RaftPeerId id, long killSleepMs, long restartSleepMs, MiniRaftCluster cluster, Logger log) {
     final CompletableFuture<Void> future = new CompletableFuture<>();
     new Thread(() -> {
       try {
         Thread.sleep(killSleepMs);
         cluster.killServer(id);
         Thread.sleep(restartSleepMs);
-        LOG.info("restart server: " + id);
+        log.info("restart server: " + id);
         cluster.restartServer(id, false);
         future.complete(null);
       } catch (Exception e) {
-        ExitUtils.terminate(-1, "Failed to kill/restart server: " + id, e, LOG);
+        ExitUtils.terminate(-1, "Failed to kill/restart server: " + id, e, log);
       }
     }).start();
     return future;
   }
 
   static void runTestBasicAppendEntries(
-      boolean async, boolean killLeader, int numMessages, MiniRaftCluster cluster, Logger LOG)
+      boolean async, boolean killLeader, int numMessages, MiniRaftCluster cluster, Logger log)
       throws Exception {
-    LOG.info("runTestBasicAppendEntries: async? {}, killLeader={}, numMessages={}",
+    log.info("runTestBasicAppendEntries: async? {}, killLeader={}, numMessages={}",
         async, killLeader, numMessages);
     for (RaftServer s : cluster.getServers()) {
       cluster.restartServer(s.getId(), false);
@@ -123,16 +125,16 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
     final long term = leader.getInfo().getCurrentTerm();
 
     final CompletableFuture<Void> killAndRestartFollower = killAndRestartServer(
-        cluster.getFollowers().get(0).getId(), 0, 1000, cluster, LOG);
+        cluster.getFollowers().get(0).getId(), 0, 1000, cluster, log);
     final CompletableFuture<Void> killAndRestartLeader;
     if (killLeader) {
-      LOG.info("killAndRestart leader " + leader.getId());
-      killAndRestartLeader = killAndRestartServer(leader.getId(), 2000, 4000, cluster, LOG);
+      log.info("killAndRestart leader " + leader.getId());
+      killAndRestartLeader = killAndRestartServer(leader.getId(), 2000, 4000, cluster, log);
     } else {
       killAndRestartLeader = CompletableFuture.completedFuture(null);
     }
 
-    LOG.info(cluster.printServers());
+    log.info(cluster.printServers());
 
     final SimpleMessage[] messages = SimpleMessage.create(numMessages);
 
@@ -152,28 +154,26 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
           });
         } else {
           final RaftClientReply reply = client.io().send(message);
-          Assert.assertTrue(reply.isSuccess());
+          Assertions.assertTrue(reply.isSuccess());
         }
       }
       if (async) {
         f.join();
-        Assert.assertEquals(messages.length, asyncReplyCount.get());
+        Assertions.assertEquals(messages.length, asyncReplyCount.get());
       }
     }
     Thread.sleep(cluster.getTimeoutMax().toIntExact(TimeUnit.MILLISECONDS) + 100);
-    LOG.info(cluster.printAllLogs());
+    log.info(cluster.printAllLogs());
     killAndRestartFollower.join();
     killAndRestartLeader.join();
 
 
     final List<RaftServer.Division> divisions = cluster.getServerAliveStream().collect(Collectors.toList());
     for(RaftServer.Division impl: divisions) {
-        JavaUtils.attempt(() -> RaftTestUtil.assertLogEntries(impl, term, messages),
-            50, TimeDuration.valueOf(1, TimeUnit.SECONDS), impl.getId() + " assertLogEntries", LOG);
+      RaftTestUtil.assertLogEntries(impl, term, messages, 50, log);
     }
   }
 
-
   @Test
   public void testOldLeaderCommit() throws Exception {
     runWithNewCluster(NUM_SERVERS, this::runTestOldLeaderCommit);
@@ -196,7 +196,7 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
     Thread.sleep(cluster.getTimeoutMax().toLong(TimeUnit.MILLISECONDS) + 100);
     for (RaftServer.Division followerToSendLog : followersToSendLog) {
       RaftLog followerLog = followerToSendLog.getRaftLog();
-      Assert.assertTrue(RaftTestUtil.logEntriesContains(followerLog, messages));
+      Assertions.assertTrue(RaftTestUtil.logEntriesContains(followerLog, messages));
     }
 
     LOG.info(String.format("killing old leader: %s", leaderId.toString()));
@@ -214,11 +214,11 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
     Set<RaftPeerId> followersToSendLogIds =
         followersToSendLog.stream().map(f -> f.getId()).collect(Collectors.toSet());
 
-    Assert.assertTrue(followersToSendLogIds.contains(newLeaderId));
+    Assertions.assertTrue(followersToSendLogIds.contains(newLeaderId));
 
     cluster.getServerAliveStream()
         .map(RaftServer.Division::getRaftLog)
-        .forEach(log -> RaftTestUtil.assertLogEntries(log, term, messages));
+        .forEach(log -> RaftTestUtil.assertLogEntries(log, term, messages, System.out::println));
   }
 
   @Test
@@ -236,8 +236,8 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
         cluster.killServer(followers.get(i).getId());
       }
     } catch (IndexOutOfBoundsException e) {
-      throw new org.junit.AssumptionViolatedException("The assumption is follower.size() = NUM_SERVERS - 1, "
-              + "actual NUM_SERVERS is " + NUM_SERVERS + ", and actual follower.size() is " + followers.size(), e);
+      Assumptions.abort("The assumption is follower.size() = NUM_SERVERS - 1, "
+          + "actual NUM_SERVERS is " + NUM_SERVERS + ", and actual follower.size() is " + followers.size());
     }
 
     SimpleMessage[] messages = SimpleMessage.create(1);
@@ -271,16 +271,16 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
     final AtomicReference<Throwable> exceptionInClientThread = new AtomicReference<>();
 
     final MiniRaftCluster cluster;
-    final Logger LOG;
+    final Logger log;
 
     Client4TestWithLoad(int index, int numMessages, boolean useAsync,
-        MiniRaftCluster cluster, Logger LOG) {
+        MiniRaftCluster cluster, Logger log) {
       super("client-" + index);
       this.index = index;
       this.messages = SimpleMessage.create(numMessages, index + "-");
       this.useAsync = useAsync;
       this.cluster = cluster;
-      this.LOG = LOG;
+      this.log = log;
     }
 
     boolean isRunning() {
@@ -295,7 +295,7 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
           if (!useAsync) {
             final RaftClientReply reply =
                 client.io().send(messages[step.getAndIncrement()]);
-            Assert.assertTrue(reply.isSuccess());
+            Assertions.assertTrue(reply.isSuccess());
           } else {
             final CompletableFuture<RaftClientReply> replyFuture =
                 client.async().send(messages[i]);
@@ -307,20 +307,20 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
               if (step.incrementAndGet() == messages.length) {
                 f.complete(null);
               }
-              Assert.assertTrue(r.isSuccess());
+              Assertions.assertTrue(r.isSuccess());
             });
           }
         }
         if (useAsync) {
           f.join();
-          Assert.assertTrue(step.get() == messages.length);
+          Assertions.assertEquals(step.get(), messages.length);
         }
       } catch(Exception t) {
         if (exceptionInClientThread.compareAndSet(null, t)) {
-          LOG.error(this + " failed", t);
+          log.error(this + " failed", t);
         } else {
           exceptionInClientThread.get().addSuppressed(t);
-          LOG.error(this + " failed again!", t);
+          log.error(this + " failed again!", t);
         }
       } finally {
         isRunning.set(false);
@@ -339,20 +339,21 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
   }
 
   @Test
+  @Timeout(value = 300)
   public void testWithLoad() throws Exception {
     runWithNewCluster(NUM_SERVERS, cluster -> testWithLoad(10, 300, false, cluster, LOG));
   }
 
   static void testWithLoad(final int numClients, final int numMessages,
-      boolean useAsync, MiniRaftCluster cluster, Logger LOG) throws Exception {
-    LOG.info("Running testWithLoad: numClients=" + numClients
+      boolean useAsync, MiniRaftCluster cluster, Logger log) throws Exception {
+    log.info("Running testWithLoad: numClients=" + numClients
         + ", numMessages=" + numMessages + ", async=" + useAsync);
 
     waitForLeader(cluster);
 
     final List<Client4TestWithLoad> clients
         = Stream.iterate(0, i -> i+1).limit(numClients)
-        .map(i -> new Client4TestWithLoad(i, numMessages, useAsync, cluster, LOG))
+        .map(i -> new Client4TestWithLoad(i, numMessages, useAsync, cluster, log))
         .collect(Collectors.toList());
     final AtomicInteger lastStep = new AtomicInteger();
 
@@ -362,24 +363,24 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
 
       @Override
       public void run() {
-        LOG.info(cluster.printServers());
-        LOG.info(BlockRequestHandlingInjection.getInstance().toString());
-        LOG.info(cluster.toString());
-        clients.forEach(c -> LOG.info("  " + c));
-        JavaUtils.dumpAllThreads(s -> LOG.info(s));
+        log.info(cluster.printServers());
+        log.info(BlockRequestHandlingInjection.getInstance().toString());
+        log.info(cluster.toString());
+        clients.forEach(c -> log.info("  " + c));
+        JavaUtils.dumpAllThreads(s -> log.info(s));
 
         final int last = lastStep.get();
         if (last != previousLastStep) {
           previousLastStep = last;
         } else {
           final RaftServer.Division leader = cluster.getLeader();
-          LOG.info("NO PROGRESS at " + last + ", try to restart leader=" + leader);
+          log.info("NO PROGRESS at " + last + ", try to restart leader=" + leader);
           if (leader != null) {
             try {
               cluster.restartServer(leader.getId(), false);
-              LOG.info("Restarted leader=" + leader);
+              log.info("Restarted leader=" + leader);
             } catch (IOException e) {
-              LOG.error("Failed to restart leader=" + leader);
+              log.error("Failed to restart leader=" + leader);
             }
           }
         }
@@ -395,7 +396,7 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
       }
 
       final int n = clients.stream().mapToInt(c -> c.step.get()).sum();
-      Assert.assertTrue(n >= lastStep.get());
+      Assertions.assertTrue(n >= lastStep.get());
 
       if (n - lastStep.get() < 50 * numClients) { // Change leader at least 50 steps.
         Thread.sleep(10);
@@ -410,10 +411,10 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
           RaftTestUtil.changeLeader(cluster, leader.getId());
         }
       } catch (IllegalStateException e) {
-        LOG.error("Failed to change leader ", e);
+        log.error("Failed to change leader ", e);
       }
     }
-    LOG.info("Leader change count=" + count);
+    log.info("Leader change count=" + count);
     timer.cancel();
 
     for(Client4TestWithLoad c : clients) {
@@ -424,7 +425,7 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
     }
   }
 
-  public static void testRequestTimeout(boolean async, MiniRaftCluster cluster, Logger LOG) throws Exception {
+  public static void testRequestTimeout(boolean async, MiniRaftCluster cluster, Logger log) throws Exception {
     waitForLeader(cluster);
     final Timestamp startTime = Timestamp.currentTime();
     try (final RaftClient client = cluster.createClient()) {
@@ -449,12 +450,16 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
       // The duration for which the client waits should be more than the retryCacheExpiryDuration.
       final TimeDuration duration = startTime.elapsedTime();
       TimeDuration retryCacheExpiryDuration = RaftServerConfigKeys.RetryCache.expiryTime(cluster.getProperties());
-      Assert.assertTrue(duration.compareTo(retryCacheExpiryDuration) >= 0);
+      Assertions.assertTrue(duration.compareTo(retryCacheExpiryDuration) >= 0);
     }
   }
 
-  public static void testStateMachineMetrics(boolean async,
-      MiniRaftCluster cluster, Logger LOG) throws Exception {
+  @Test
+  public void testStateMachineMetrics() throws Exception {
+    runWithNewCluster(NUM_SERVERS, cluster -> runTestStateMachineMetrics(false, cluster));
+  }
+
+  static void runTestStateMachineMetrics(boolean async, MiniRaftCluster cluster) throws Exception {
     RaftServer.Division leader = waitForLeader(cluster);
     try (final RaftClient client = cluster.createClient()) {
       Gauge appliedIndexGauge = getStatemachineGaugeWithName(leader,
@@ -477,10 +482,10 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
       long smAppliedIndexAfter = (Long) smAppliedIndexGauge.getValue();
       checkFollowerCommitLagsLeader(cluster);
 
-      Assert.assertTrue("StateMachine Applied Index not incremented",
-          appliedIndexAfter > appliedIndexBefore);
-      Assert.assertTrue("StateMachine Apply completed Index not incremented",
-          smAppliedIndexAfter > smAppliedIndexBefore);
+      Assertions.assertTrue(appliedIndexAfter > appliedIndexBefore,
+          "StateMachine Applied Index not incremented");
+      Assertions.assertTrue(smAppliedIndexAfter > smAppliedIndexBefore,
+          "StateMachine Apply completed Index not incremented");
     }
   }
 
@@ -493,12 +498,12 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
     for (RaftServer.Division f : followers) {
       final RaftGroupMemberId follower = f.getMemberId();
       Gauge followerCommitGauge = ServerMetricsTestUtils.getPeerCommitIndexGauge(leader, follower.getPeerId());
-      Assert.assertTrue((Long)leaderCommitGauge.getValue() >=
+      Assertions.assertTrue((Long)leaderCommitGauge.getValue() >=
           (Long)followerCommitGauge.getValue());
       Gauge followerMetric = ServerMetricsTestUtils.getPeerCommitIndexGauge(follower, follower.getPeerId());
       System.out.println(followerCommitGauge.getValue());
       System.out.println(followerMetric.getValue());
-      Assert.assertTrue((Long)followerCommitGauge.getValue()  <= (Long)followerMetric.getValue());
+      Assertions.assertTrue((Long)followerCommitGauge.getValue()  <= (Long)followerMetric.getValue());
     }
   }
 
@@ -509,7 +514,7 @@ public abstract class RaftBasicTests<CLUSTER extends MiniRaftCluster>
         RATIS_STATEMACHINE_METRICS, RATIS_STATEMACHINE_METRICS_DESC);
 
     Optional<RatisMetricRegistry> metricRegistry = MetricRegistries.global().get(info);
-    Assert.assertTrue(metricRegistry.isPresent());
+    Assertions.assertTrue(metricRegistry.isPresent());
 
     return ServerMetricsTestUtils.getGaugeWithName(gaugeName, metricRegistry::get);
   }
