@@ -44,6 +44,7 @@ import org.apache.ratis.util.ProtoUtils;
 import org.apache.ratis.util.TimeDuration;
 import org.junit.Assert;
 import org.junit.AssumptionViolatedException;
+import org.junit.jupiter.api.Assertions;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -51,8 +52,12 @@ import java.io.IOException;
 import java.lang.reflect.Field;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collections;
+import java.util.Comparator;
 import java.util.EnumMap;
+import java.util.HashMap;
 import java.util.List;
+import java.util.Map;
 import java.util.Objects;
 import java.util.Optional;
 import java.util.concurrent.CompletableFuture;
@@ -248,19 +253,16 @@ public interface RaftTestUtil {
     }
   }
 
-  static void assertLogEntries(RaftServer.Division server, long expectedTerm, SimpleMessage... expectedMessages) {
-    LOG.info("checking raft log for {}", server.getMemberId());
-    final RaftLog log = server.getRaftLog();
-    try {
-      RaftTestUtil.assertLogEntries(log, expectedTerm, expectedMessages);
-    } catch (AssertionError e) {
-      LOG.error("Unexpected raft log in {}", server.getMemberId(), e);
-      throw e;
-    }
+  static void assertLogEntries(RaftServer.Division server, long expectedTerm, SimpleMessage[] expectedMessages,
+      int numAttempts, Logger log) throws Exception {
+    final String name = server.getId() + " assertLogEntries";
+    final Function<Integer, Consumer<String>> print = i -> i < numAttempts? s -> {}: System.out::println;
+    JavaUtils.attempt(i -> assertLogEntries(server.getRaftLog(), expectedTerm, expectedMessages, print.apply(i)),
+        numAttempts, TimeDuration.ONE_SECOND, () -> name, log);
   }
 
   static Iterable<LogEntryProto> getLogEntryProtos(RaftLog log) {
-    return CollectionUtils.as(log.getEntries(0, Long.MAX_VALUE), ti -> {
+    return CollectionUtils.as(log.getEntries(0, log.getLastEntryTermIndex().getIndex() + 1), ti -> {
       try {
         return log.get(ti.getIndex());
       } catch (IOException exception) {
@@ -269,17 +271,54 @@ public interface RaftTestUtil {
     });
   }
 
-  static List<LogEntryProto> getStateMachineLogEntries(RaftLog log) {
+  Comparator<LogEntryProto> LOG_ENTRY_PROTO_COMPARATOR = Comparator.comparing(
+      e -> e.getStateMachineLogEntry().getLogData().asReadOnlyByteBuffer());
+  Comparator<SimpleMessage> SIMPLE_MESSAGE_COMPARATOR = Comparator.comparing(
+      m -> m.getContent().asReadOnlyByteBuffer());
+
+  /** @return a map of message-array-index to {@link LogEntryProto} for the entries found in the given log. */
+  static Map<Integer, LogEntryProto> getStateMachineLogEntries(RaftLog log, SimpleMessage[] messages) {
+    if (messages.length == 0) {
+      return Collections.emptyMap();
+    }
+    final List<LogEntryProto> entries = getStateMachineLogEntries(log, s -> {});
+    if (entries.isEmpty()) {
+      return Collections.emptyMap();
+    }
+
+    entries.sort(LOG_ENTRY_PROTO_COMPARATOR);
+    Arrays.sort(messages, SIMPLE_MESSAGE_COMPARATOR);
+
+    final Map<Integer, LogEntryProto> found = new HashMap<>();
+    for (int e = 0, m = 0; e < entries.size() && m < messages.length; ) {
+      final int diff = messages[m].getContent().asReadOnlyByteBuffer().compareTo(
+          entries.get(e).getStateMachineLogEntry().getLogData().asReadOnlyByteBuffer());
+      if (diff == 0) {
+        found.put(m, entries.get(e));
+        m++;
+        e++;
+      } else if (diff < 0) {
+        m++; // message < entry
+      } else {
+        e++; // message > entry
+      }
+    }
+
+    Assertions.assertEquals(messages.length, found.size());
+    return found;
+  }
+
+  static List<LogEntryProto> getStateMachineLogEntries(RaftLog log, Consumer<String> print) {
     final List<LogEntryProto> entries = new ArrayList<>();
     for (LogEntryProto e : getLogEntryProtos(log)) {
       final String s = LogProtoUtils.toLogEntryString(e);
       if (e.hasStateMachineLogEntry()) {
-        LOG.info(s + ", " + e.getStateMachineLogEntry().toString().trim().replace("\n", ", "));
+        print.accept(entries.size() + ") " + s);
         entries.add(e);
       } else if (e.hasConfigurationEntry()) {
-        LOG.info("Found {}, ignoring it.", s);
+        print.accept("Ignoring " + s);
       } else if (e.hasMetadataEntry()) {
-        LOG.info("Found {}, ignoring it.", s);
+        print.accept("Ignoring " + s);
       } else {
         throw new AssertionError("Unexpected LogEntryBodyCase " + e.getLogEntryBodyCase() + " at " + s);
       }
@@ -287,13 +326,15 @@ public interface RaftTestUtil {
     return entries;
   }
 
-  static void assertLogEntries(RaftLog log, long expectedTerm, SimpleMessage... expectedMessages) {
-    final List<LogEntryProto> entries = getStateMachineLogEntries(log);
+  static Void assertLogEntries(RaftLog log, long expectedTerm, SimpleMessage[] expectedMessages,
+      Consumer<String> print) {
+    final List<LogEntryProto> entries = getStateMachineLogEntries(log, print);
     try {
       assertLogEntries(entries, expectedTerm, expectedMessages);
     } catch(Exception t) {
       throw new AssertionError("entries: " + entries, t);
     }
+    return null;
   }
 
   static void assertLogEntries(List<LogEntryProto> entries, long expectedTerm, SimpleMessage... expectedMessages) {
@@ -307,8 +348,7 @@ public interface RaftTestUtil {
       }
       Assert.assertTrue(e.getIndex() > logIndex);
       logIndex = e.getIndex();
-      Assert.assertArrayEquals(expectedMessages[i].getContent().toByteArray(),
-          e.getStateMachineLogEntry().getLogData().toByteArray());
+      Assert.assertEquals(expectedMessages[i].getContent(), e.getStateMachineLogEntry().getLogData());
     }
   }
 
@@ -366,8 +406,8 @@ public interface RaftTestUtil {
   }
 
   class SimpleOperation {
-    private static final ClientId clientId = ClientId.randomId();
-    private static final AtomicLong callId = new AtomicLong();
+    private static final ClientId CLIENT_ID = ClientId.randomId();
+    private static final AtomicLong CALL_ID = new AtomicLong();
 
     private final String op;
     private final StateMachineLogEntryProto smLogEntryProto;
@@ -377,7 +417,7 @@ public interface RaftTestUtil {
     }
 
     public SimpleOperation(String op, boolean hasStateMachineData) {
-      this(clientId, callId.incrementAndGet(), op, hasStateMachineData);
+      this(CLIENT_ID, CALL_ID.incrementAndGet(), op, hasStateMachineData);
     }
 
     private SimpleOperation(ClientId clientId, long callId, String op, boolean hasStateMachineData) {
@@ -472,6 +512,20 @@ public interface RaftTestUtil {
     Thread.sleep(3 * maxTimeout.toLong(TimeUnit.MILLISECONDS));
   }
 
+  static void isolate(MiniRaftCluster cluster, RaftPeerId id) {
+    try {
+      BlockRequestHandlingInjection.getInstance().blockReplier(id.toString());
+      cluster.setBlockRequestsFrom(id.toString(), true);
+    } catch (Exception e) {
+      e.printStackTrace();
+    }
+  }
+
+  static void deIsolate(MiniRaftCluster cluster, RaftPeerId id) {
+    BlockRequestHandlingInjection.getInstance().unblockReplier(id.toString());
+    cluster.setBlockRequestsFrom(id.toString(), false);
+  }
+
   static Thread sendMessageInNewThread(MiniRaftCluster cluster, RaftPeerId leaderId, SimpleMessage... messages) {
     Thread t = new Thread(() -> {
       try (final RaftClient client = cluster.createClient(leaderId)) {
