@@ -31,8 +31,10 @@ import org.apache.ratis.retry.RetryPolicies;
 import org.apache.ratis.retry.RetryPolicy;
 import org.apache.ratis.server.RaftServer;
 import org.apache.ratis.server.RaftServerConfigKeys;
+import org.apache.ratis.server.RaftServerConfigKeys.Watch;
 import org.apache.ratis.server.impl.MiniRaftCluster;
 import org.apache.ratis.server.impl.RaftServerTestUtil;
+import org.apache.ratis.server.metrics.RaftServerMetricsImpl;
 import org.apache.ratis.statemachine.impl.SimpleStateMachine4Testing;
 import org.apache.ratis.statemachine.StateMachine;
 import org.apache.ratis.util.Slf4jUtils;
@@ -139,26 +141,26 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
     }
   }
 
-  static void runTest(CheckedConsumer<TestParameters, Exception> testCase, MiniRaftCluster cluster, Logger LOG)
+  static void runTest(CheckedConsumer<TestParameters, Exception> testCase, MiniRaftCluster cluster, Logger log)
       throws Exception {
     try(final RaftClient client = cluster.createClient(RaftTestUtil.waitForLeader(cluster).getId())) {
       final int[] numMessages = {1, 10, 20};
       for(int n : numMessages) {
-        final TestParameters p = new TestParameters(n, client, cluster, LOG);
-        LOG.info("{}) {}, {}", n, p, cluster.printServers());
+        final TestParameters p = new TestParameters(n, client, cluster, log);
+        log.info("{}) {}, {}", n, p, cluster.printServers());
         testCase.accept(p);
       }
     }
   }
 
   static void runSingleTest(CheckedConsumer<TestParameters, Exception> testCase,
-      MiniRaftCluster cluster, Logger LOG)
+      MiniRaftCluster cluster, Logger log)
       throws Exception {
     try(final RaftClient client = cluster.createClient(RaftTestUtil.waitForLeader(cluster).getId())) {
       final int[] numMessages = {1};
       for(int n : numMessages) {
-        final TestParameters p = new TestParameters(n, client, cluster, LOG);
-        LOG.info("{}) {}, {}", n, p, cluster.printServers());
+        final TestParameters p = new TestParameters(n, client, cluster, log);
+        log.info("{}) {}, {}", n, p, cluster.printServers());
         testCase.accept(p);
       }
     }
@@ -174,7 +176,8 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
 
     WatchReplies(long logIndex,
         CompletableFuture<RaftClientReply> majority, CompletableFuture<RaftClientReply> all,
-        CompletableFuture<RaftClientReply> majorityCommitted, CompletableFuture<RaftClientReply> allCommitted, Logger log) {
+        CompletableFuture<RaftClientReply> majorityCommitted, CompletableFuture<RaftClientReply> allCommitted,
+        Logger log) {
       this.logIndex = logIndex;
       this.majority = majority;
       this.all = all;
@@ -216,19 +219,19 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
   }
 
   static void runTestWatchRequestAsync(TestParameters p) throws Exception {
-    final Logger LOG = p.log;
+    final Logger log = p.log;
     final MiniRaftCluster cluster = p.cluster;
     final int numMessages = p.numMessages;
 
     // blockStartTransaction of the leader so that no transaction can be committed MAJORITY
     final RaftServer.Division leader = cluster.getLeader();
-    LOG.info("block leader {}", leader.getId());
+    log.info("block leader {}", leader.getId());
     SimpleStateMachine4Testing.get(leader).blockStartTransaction();
 
     // blockFlushStateMachineData a follower so that no transaction can be ALL_COMMITTED
     final List<RaftServer.Division> followers = cluster.getFollowers();
     final RaftServer.Division blockedFollower = followers.get(ThreadLocalRandom.current().nextInt(followers.size()));
-    LOG.info("block follower {}", blockedFollower.getId());
+    log.info("block follower {}", blockedFollower.getId());
     SimpleStateMachine4Testing.get(blockedFollower).blockFlushStateMachineData();
 
     // send a message
@@ -247,9 +250,9 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
 
     // unblock leader so that the transaction can be committed.
     SimpleStateMachine4Testing.get(leader).unblockStartTransaction();
-    LOG.info("unblock leader {}", leader.getId());
+    log.info("unblock leader {}", leader.getId());
 
-    checkMajority(replies, watches, LOG);
+    checkMajority(replies, watches, log);
 
     Assert.assertEquals(numMessages, watches.size());
 
@@ -259,16 +262,16 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
     assertNotDone(watches.stream().map(CompletableFuture::join).map(w -> w.allCommitted));
 
     // unblock follower so that the transaction can be replicated and committed to all.
-    LOG.info("unblock follower {}", blockedFollower.getId());
+    log.info("unblock follower {}", blockedFollower.getId());
     SimpleStateMachine4Testing.get(blockedFollower).unblockFlushStateMachineData();
-    checkAll(watches, LOG);
+    checkAll(watches, log);
   }
 
   static void checkMajority(List<CompletableFuture<RaftClientReply>> replies,
-      List<CompletableFuture<WatchReplies>> watches, Logger LOG) throws Exception {
+      List<CompletableFuture<WatchReplies>> watches, Logger log) throws Exception {
     for(int i = 0; i < replies.size(); i++) {
       final RaftClientReply reply = replies.get(i).get(GET_TIMEOUT_SECOND, TimeUnit.SECONDS);
-      LOG.info("checkMajority {}: receive {}", i, reply);
+      log.info("checkMajority {}: receive {}", i, reply);
       final long logIndex = reply.getLogIndex();
       Assert.assertTrue(reply.isSuccess());
 
@@ -294,11 +297,11 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
     }
   }
 
-  static void checkAll(List<CompletableFuture<WatchReplies>> watches, Logger LOG) throws Exception {
+  static void checkAll(List<CompletableFuture<WatchReplies>> watches, Logger log) throws Exception {
     for(int i = 0; i < watches.size(); i++) {
       final WatchReplies watchReplies = watches.get(i).get(GET_TIMEOUT_SECOND, TimeUnit.SECONDS);
       final long logIndex = watchReplies.logIndex;
-      LOG.info("checkAll {}: logIndex={}", i, logIndex);
+      log.info("checkAll {}: logIndex={}", i, logIndex);
       final RaftClientReply watchAllReply = watchReplies.getAll();
 
       final RaftClientReply watchAllCommittedReply = watchReplies.getAllCommitted();
@@ -334,14 +337,14 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
   }
 
   static void runTestWatchRequestAsyncChangeLeader(TestParameters p) throws Exception {
-    final Logger LOG = p.log;
+    final Logger log = p.log;
     final MiniRaftCluster cluster = p.cluster;
     final int numMessages = p.numMessages;
 
     // blockFlushStateMachineData a follower so that no transaction can be ALL_COMMITTED
     final List<RaftServer.Division> followers = cluster.getFollowers();
     final RaftServer.Division blockedFollower = followers.get(ThreadLocalRandom.current().nextInt(followers.size()));
-    LOG.info("block follower {}", blockedFollower.getId());
+    log.info("block follower {}", blockedFollower.getId());
     SimpleStateMachine4Testing.get(blockedFollower).blockFlushStateMachineData();
 
     final List<CompletableFuture<RaftClientReply>> replies = new ArrayList<>();
@@ -353,7 +356,7 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
     Assert.assertEquals(numMessages, watches.size());
 
     // since only one follower is blocked commit, requests can be committed MAJORITY and ALL but not ALL_COMMITTED.
-    checkMajority(replies, watches, LOG);
+    checkMajority(replies, watches, log);
 
     TimeUnit.SECONDS.sleep(1);
     assertNotDone(watches.stream().map(CompletableFuture::join).map(w -> w.allCommitted));
@@ -363,8 +366,8 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
 
     // unblock follower so that the transaction can be replicated and committed to all.
     SimpleStateMachine4Testing.get(blockedFollower).unblockFlushStateMachineData();
-    LOG.info("unblock follower {}", blockedFollower.getId());
-    checkAll(watches, LOG);
+    log.info("unblock follower {}", blockedFollower.getId());
+    checkAll(watches, log);
   }
 
   @Test
@@ -382,7 +385,7 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
   }
 
   static void runTestWatchRequestTimeout(TestParameters p) throws Exception {
-    final Logger LOG = p.log;
+    final Logger log = p.log;
     final MiniRaftCluster cluster = p.cluster;
     final int numMessages = p.numMessages;
 
@@ -392,13 +395,13 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
 
     // blockStartTransaction of the leader so that no transaction can be committed MAJORITY
     final RaftServer.Division leader = cluster.getLeader();
-    LOG.info("block leader {}", leader.getId());
+    log.info("block leader {}", leader.getId());
     SimpleStateMachine4Testing.get(leader).blockStartTransaction();
 
     // blockFlushStateMachineData a follower so that no transaction can be ALL_COMMITTED
     final List<RaftServer.Division> followers = cluster.getFollowers();
     final RaftServer.Division blockedFollower = followers.get(ThreadLocalRandom.current().nextInt(followers.size()));
-    LOG.info("block follower {}", blockedFollower.getId());
+    log.info("block follower {}", blockedFollower.getId());
     SimpleStateMachine4Testing.get(blockedFollower).blockFlushStateMachineData();
 
     // send a message
@@ -417,13 +420,13 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
 
     // unblock leader so that the transaction can be committed.
     SimpleStateMachine4Testing.get(leader).unblockStartTransaction();
-    LOG.info("unblock leader {}", leader.getId());
+    log.info("unblock leader {}", leader.getId());
 
-    checkMajority(replies, watches, LOG);
-    checkTimeout(replies, watches, LOG);
+    checkMajority(replies, watches, log);
+    checkTimeout(replies, watches, log);
 
     SimpleStateMachine4Testing.get(blockedFollower).unblockFlushStateMachineData();
-    LOG.info("unblock follower {}", blockedFollower.getId());
+    log.info("unblock follower {}", blockedFollower.getId());
   }
 
   @Test
@@ -444,7 +447,7 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
   }
 
   static void runTestWatchRequestClientTimeout(TestParameters p) throws Exception {
-    final Logger LOG = p.log;
+    final Logger log = p.log;
 
     CompletableFuture<RaftClientReply> watchReply;
     // watch 1000 which will never be committed
@@ -457,7 +460,7 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
       watchReply.get();
       fail("runTestWatchRequestClientTimeout failed");
     } catch (Exception ex) {
-      LOG.error("error occurred", ex);
+      log.error("error occurred", ex);
       Assert.assertTrue(ex.getCause().getClass() == AlreadyClosedException.class ||
           ex.getCause().getClass() == RaftRetryFailureException.class);
       if (ex.getCause() != null) {
@@ -469,11 +472,64 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
     }
   }
 
+  @Test
+  public void testWatchMetrics() throws Exception {
+    final RaftProperties p = getProperties();
+    RaftServerConfigKeys.Watch.setElementLimit(p, 10);
+    RaftServerConfigKeys.Watch.setTimeout(p, TimeDuration.valueOf(2, TimeUnit.SECONDS));
+    try {
+      runWithNewCluster(NUM_SERVERS,
+          cluster -> runSingleTest(WatchRequestTests::runTestWatchMetrics, cluster, LOG));
+    } finally {
+      RaftServerConfigKeys.Watch.setElementLimit(p, Watch.ELEMENT_LIMIT_DEFAULT);
+      RaftServerConfigKeys.Watch.setTimeout(p, RaftServerConfigKeys.Watch.TIMEOUT_DEFAULT);
+    }
+  }
+
+  static RaftServerMetricsImpl getRaftServerMetrics(RaftServer.Division division) {
+    return (RaftServerMetricsImpl) division.getRaftServerMetrics();
+  }
+
+  static void runTestWatchMetrics(TestParameters p) throws Exception {
+    final MiniRaftCluster cluster = p.cluster;
+
+    List<RaftClient> clients = new ArrayList<>();
+
+    final ReplicationLevel replicationLevel = ReplicationLevel.MAJORITY;
+    try {
+      long initialWatchRequestTimeoutCount = getRaftServerMetrics(cluster.getLeader())
+          .getNumWatchRequestsTimeout(replicationLevel).getCount();
+      long initialLimitHit = getRaftServerMetrics(cluster.getLeader())
+          .getNumWatchRequestQueueLimitHits(replicationLevel).getCount();
+
+      int uncommittedBaseIndex = 10000;
+      // Logs with indices 10001 - 10011 will never be committed, so it should fail with NotReplicatedException
+      for (int i = 1; i <= 11; i++) {
+        RaftClient client = cluster.createClient(cluster.getLeader().getId(), RetryPolicies.noRetry());
+        clients.add(client);
+        client.async().watch(uncommittedBaseIndex + i, replicationLevel);
+      }
+
+      // All the watch timeout for each unique index should increment the metric
+      RaftTestUtil.waitFor(() -> getRaftServerMetrics(cluster.getLeader())
+              .getNumWatchRequestsTimeout(replicationLevel).getCount() == initialWatchRequestTimeoutCount + 10,
+          300, 5000);
+      // There are 11 pending watch request, but the pending watch request limit is 10
+      RaftTestUtil.waitFor(() -> getRaftServerMetrics(cluster.getLeader())
+          .getNumWatchRequestQueueLimitHits(replicationLevel).getCount() ==
+          initialLimitHit + 1, 300, 5000);
+    } finally {
+      for(RaftClient client : clients) {
+        client.close();
+      }
+    }
+  }
+
   static void checkTimeout(List<CompletableFuture<RaftClientReply>> replies,
-      List<CompletableFuture<WatchReplies>> watches, Logger LOG) throws Exception {
+      List<CompletableFuture<WatchReplies>> watches, Logger log) throws Exception {
     for(int i = 0; i < replies.size(); i++) {
       final RaftClientReply reply = replies.get(i).get(GET_TIMEOUT_SECOND, TimeUnit.SECONDS);
-      LOG.info("checkTimeout {}: receive {}", i, reply);
+      log.info("checkTimeout {}: receive {}", i, reply);
       final long logIndex = reply.getLogIndex();
       Assert.assertTrue(reply.isSuccess());
 
@@ -503,5 +559,6 @@ public abstract class WatchRequestTests<CLUSTER extends MiniRaftCluster>
     Assert.assertNotNull(nre);
     Assert.assertEquals(logIndex, nre.getLogIndex());
     Assert.assertEquals(replication, nre.getRequiredReplication());
+    Assert.assertNotNull(nre.getCommitInfos());
   }
 }
